/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */

#include <sal/config.h>

#include <com/sun/star/uno/Reference.hxx>
#include <com/sun/star/frame/XFrame.hpp>
#include <framework/sfxhelperfunctions.hxx>

#include <vcl/svapp.hxx>

#include <fwkhelper.hxx>
#include <workwin.hxx>
#include <sfx2/frame.hxx>

void RefreshToolbars( css::uno::Reference< css::frame::XFrame > const & xFrame )
{
    SolarMutexGuard aGuard;
    if ( !xFrame.is() )
        return;

    SfxFrame* pFrame=nullptr;
    for ( pFrame = SfxFrame::GetFirst(); pFrame; pFrame = SfxFrame::GetNext( *pFrame ) )
    {
        if ( pFrame->GetFrameInterface() == xFrame )
            break;
    }

    if ( pFrame )
    {
        SfxWorkWindow* pWrkWin = pFrame->GetWorkWindow_Impl();
        if ( pWrkWin )
            pWrkWin->UpdateObjectBars_Impl();
    }
}

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
